
#include <stdbool.h>
#include <string.h>
#include "app_button.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "app_timer.h"
#include "app_pwm.h"
#include "nrf_delay.h"
#include "app_uart.h"
#include "app_trace.h"

#define APP_TIMER_PRESCALER             0  // Value of the RTC1 PRESCALER register.
#define APP_TIMER_MAX_TIMERS            2  // Maximum number of simultaneously created timers. // NORDIC One for debounce, one for long press detect
#define APP_TIMER_OP_QUEUE_SIZE         2  // Size of timer operation queues. 
#define BUTTON_DEBOUNCE_DELAY		        50 // Delay from a GPIOTE event until a button is reported as pushed. 

#define LONG_BUTTON_PRESS_TIME_MS       5000
#define BUTTON_PRESS_TIME_MS            3000

#define MS_TO_TICKS(ms)                 (ms*32768/1000)
static app_timer_id_t                   m_timer_id;


#define PWM_OUTPUT1 7
#define PWM_OUTPUT2 6

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                           /**< UART RX buffer size. */
APP_PWM_INSTANCE(PWM1,1);                   // Create the instance "PWM1" using TIMER1.

static volatile bool ready_flag;            // A flag indicating PWM status.

void pwm_ready_callback(uint32_t pwm_id)    // PWM callback function
{
    ready_flag = true;
}
void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}
/*
 * Handler to be called when button is pushed.
 * param[in]   pin_no 			The pin number where the event is genereated
 * param[in]   button_action 	Is the button pushed or released
 */
static void button_handler(uint8_t pin_no, uint8_t button_action)
{
    if(button_action == APP_BUTTON_PUSH)
    {
        switch(pin_no)
        {			
            case BUTTON_1:				
                printf("button is pressed for 3 sec:\n");
                app_timer_start(m_timer_id, MS_TO_TICKS(BUTTON_PRESS_TIME_MS), NULL); // NORDIC: Start single shot timer
						    break;
                
            case BUTTON_2:
							  printf("button is pressed for 5 sec:\n");
							  app_timer_start(m_timer_id, MS_TO_TICKS(LONG_BUTTON_PRESS_TIME_MS), NULL); 
						    break;
						
           default:
                break;
        }                                                          
    }
}

/**  
 * Initialize the clock.
 */
void init_clock()
{
	
	  printf(" CLOCK initialization:\n");
    NRF_CLOCK->LFCLKSRC            = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_LFCLKSTART    = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0); // Wait for clock to start
}	

/**
 * Initialize all four LEDs on the nRF51 DK.
 */
void init_leds()
{
	   printf("LED initialization:\n");
    nrf_gpio_range_cfg_input(BUTTON_1, BUTTON_2, NRF_GPIO_PIN_PULLUP);//
    nrf_gpio_range_cfg_output(LED_3, LED_4); 
	  nrf_gpio_pin_set(LED_3); //clear the LED                              
    nrf_gpio_pin_set(LED_4);//clear the LED
                                                                                                                                  
}	
/**
 * Initialize all four LEDs on the nRF51 DK.
 */
void init_gpio()
{
    printf("gpio initialization:\n");
		nrf_gpio_range_cfg_output(6,7);
	  nrf_gpio_pin_dir_set(6, NRF_GPIO_PIN_DIR_OUTPUT);
	  nrf_gpio_pin_dir_set(7, NRF_GPIO_PIN_DIR_OUTPUT);
	  NRF_GPIO->PIN_CNF[6] &= ~(GPIO_PIN_CNF_DRIVE_Msk); // reset drive setting
    NRF_GPIO->PIN_CNF[6] = (GPIO_PIN_CNF_DRIVE_H0H1 << GPIO_PIN_CNF_DRIVE_Pos);
                     
	  NRF_GPIO->PIN_CNF[7] &= ~(GPIO_PIN_CNF_DRIVE_Msk); // reset drive setting
    NRF_GPIO->PIN_CNF[7] = (GPIO_PIN_CNF_DRIVE_H0H1 << GPIO_PIN_CNF_DRIVE_Pos);
                      
    nrf_gpio_pin_set(6);
	  nrf_gpio_pin_set(7);                                                                                                                                  
}	

static void PWMinit(void)
{
	 printf("PWM initialization:\n");
	 ret_code_t err_code;
   app_pwm_config_t pwm1_cfg = APP_PWM_DEFAULT_CONFIG_2CH(100UL,6,7);//6.3KHz
	 /* Switch the polarity of the second or first  channel. */
  // pwm1_cfg.pin_polarity[1] = APP_PWM_POLARITY_ACTIVE_HIGH;
	 //pwm1_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_HIGH;
	
	/* Initialize and enable PWM. */
    err_code = app_pwm_init(&PWM1,&pwm1_cfg,pwm_ready_callback);
    APP_ERROR_CHECK(err_code);
	    //app_pwm_enable(&PWM1);
}
void pwm_start(void)
{   
  
   printf("PWM started:\n");
	 app_pwm_enable(&PWM1);
	 uint32_t   value = 50;
   for (uint8_t i = 0; i <3; ++i)
	{
            ready_flag = false;
					 app_pwm_channel_duty_set(&PWM1, 0, value);
           nrf_delay_ms(1000);
          app_pwm_channel_duty_set(&PWM1, 0, 0);
		      nrf_delay_ms(2000);
          app_pwm_channel_duty_set(&PWM1, 1, value);
					nrf_delay_ms(1000);
		    
}
  app_pwm_disable(&PWM1);  
  nrf_gpio_pin_clear(6);
  nrf_gpio_pin_clear(7);
}
 void pwm_stop(void)
{
	printf("PWM stoped:\n");
	app_pwm_channel_duty_set(&PWM1, 0, 0);
	app_pwm_channel_duty_set(&PWM1, 1, 0);  
  app_pwm_disable(&PWM1);
 
}
/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler(void * p_context)
{
     if (!nrf_gpio_pin_read(BUTTON_1))
		{
			 
        nrf_gpio_pin_clear(LED_3);  // Turn  LED 1 if button 1 is still pressed
			  pwm_start();
			  nrf_gpio_pin_clear(LED_4);
		}
    else
		{ 
        if(!nrf_gpio_pin_read(BUTTON_2)) 
			  nrf_gpio_pin_set(LED_3); // Turn off LED 1 if button 2 is still pressed
			  pwm_stop();
				nrf_gpio_pin_set(LED_4);
				
			}		 
}
/**@brief Function for initializing the button handler module.
 */
static void buttons_init(void)
{
	
	   printf("BUTTON initialization:\n");
     uint32_t err_code;
    // Button configuration structure.
    static app_button_cfg_t p_button[] = {  {BUTTON_1, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler},
                                          {BUTTON_2, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler}};
        
    // Macro for initializing the application timer module.
    // It will handle dimensioning and allocation of the memory buffer required by the timer, making sure that the buffer is correctly aligned. It will also connect the timer module to the scheduler (if specified).
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, NULL);    
    app_timer_create(&m_timer_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler); // NORDIC Create single shot timer triggering interrupt handler timer_handler once
    
    // Initializing the buttons.
    err_code = app_button_init(p_button, sizeof(p_button) / sizeof(p_button[0]), BUTTON_DEBOUNCE_DELAY);
    APP_ERROR_CHECK(err_code);
                                            
    // Enabling the buttons.										
    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code);
		while(true)
    {
        // Do nothing.
    }
}

/**
 * Function for application main entry.
 */
int main(void)
{	uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          RX_PIN_NUMBER,
          TX_PIN_NUMBER,
          RTS_PIN_NUMBER,
          CTS_PIN_NUMBER,
          APP_UART_FLOW_CONTROL_ENABLED,
          false,
          UART_BAUDRATE_BAUDRATE_Baud38400
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOW,
                         err_code);

    APP_ERROR_CHECK(err_code);
		printf("initialization all functions :\n");
    init_leds();
    init_clock();
		init_gpio();
	  PWMinit();
	 
    buttons_init();
	  
    
}

